
/* Jenny (Craig) 1.0

   Created 7/23/93

   Strips a NS 3.x fat binary which contains a Motorola NS 2.x runnable
   executeable down to a NS 2.x runnable executeable.

   By Christoper Lloyd, Yrrid Inc.
   lloyd@world.std.com
 
   This program is in the public domain.

   To build:

     cc jenny1.0.c -o jenny; strip jenny

   To use:

     - Build Motorola executeable under NS 2.x, which runs properly on 3.x.
     - Build Intel executeable under NS 3.x.
     - Using lipo under 3.x:
         lipo -create <intel> <motorola> -output <final-binary>

     The final fat binary is suitable for running under NS 3.x.

     AND

     jenny <executeable>

     If you run jenny on the executeable, it will strip it down to
     the original Motorola 2.x version, suitable for use under NS 2.x.

   --

   I have tested this very little, but there is nothing magic about
   what's happening, so I wouldn't expect it to be problematic.

 */



#import <streams/streams.h>
#import <stdio.h>
#import <stdlib.h>

#import <sys/loader.h>

/* some fat definitions stolen from 3.1 */

#define FAT_MAGIC	0xcafebabe

struct fat_arch {
   cpu_type_t    cputype;
   cpu_subtype_t cpusubtype;
   unsigned long offset;
   unsigned long size;
   unsigned long align;
};

struct fat_header {
   unsigned long   magic;
   unsigned long   nfat_arch;
   struct fat_arch archs[0];
};


static void fatal(const char *fmt,...)
{
   va_list list;
   va_start(list,fmt);

   vfprintf(stderr,fmt,list);
   fprintf(stderr,".\n");
   exit(-1);
}

void main(int argc,char **argv)
{
   NXStream *in;
   NXStream *out;
   char *inData;
   int   inLen,inMaxLen;

   if(argc<2)
    fatal("No input file specified");

   if((in=NXMapFile(argv[1],NX_READONLY))==NULL)
    fatal("Cannot open file %s",argv[1]);

   if((out=NXOpenMemory(NULL,0,NX_READWRITE))==NULL)
    fatal("Cannot open memory");

   NXGetMemoryBuffer(in,&inData,&inLen,&inMaxLen);

   if(inLen<sizeof(struct fat_header))
    fatal("File too short");
   else
   {
    struct fat_header *fat=(struct fat_header *)inData;

    if(fat->magic!=FAT_MAGIC)
     fatal("Not a fat file");
    else{
     int i;

     for(i=0;i<fat->nfat_arch;i++)
      if(fat->archs[i].cputype==CPU_TYPE_MC680x0)
       break;

     if(i>=fat->nfat_arch)
      fatal("No Motorola 680x0 executeable found.");
     else{
      NXWrite(out,inData+fat->archs[i].offset,fat->archs[i].size);
      NXSaveToFile(out,argv[1]);
     }
    }
   }
}


